#ifndef __DEMOLINO_H__
#define __DEMOLINO_H__

#include <GL/glut.h> 
#include <stdlib.h> 
#include <stdio.h>

// Type
#define float GLfloat

// Uilities
#define CLEAR glClear(GL_COLOR_BUFFER_BIT)
#define RGB(R, G, B) (float)(R)/255.0f, (float)(G)/255.0f, (float)(B)/255.0f
#define RGBA(R, G, B, A) RGB(R, G, B), (float)(A)/255.0f
#define SET_COLOR_RGB(R, G, B) glColor3f(RGB(R, G, B))
#define SET_COLOR_RGBA(R, G, B, A) glColor4f(RGBA(R, G, B, A))
#define PUSH_MAT glPushMatrix()
#define POP_MAT glPopMatrix()
#define SWAP glutSwapBuffers()

// Custom info
#ifndef DEMO_TITLE
#define DEMO_TITLE "Demolino"
#endif

#ifndef DEMO_WIN_SIZE
#define DEMO_WIN_SIZE 320,240
#endif

#ifndef DEMO_FPS
#define DEMO_FPS 30
#endif

#ifndef DEMO_CLEAR_COLOR
#define DEMO_CLEAR_COLOR RGBA(0, 0, 0, 0)
#endif

void timer(int value);
void render(void);
void init(void);

// Called by GLUT library when idle (window not being
// resized or moved)
void TimerFunction(int value)
{
	timer(value);

	glutPostRedisplay();
	glutTimerFunc(1000/DEMO_FPS, TimerFunction, 1);
}

void SpecialKeys(int key, int x, int y)
{
	/*
	if(key == GLUT_KEY_UP) xRot-= 5.0f;
	if(key == GLUT_KEY_DOWN) xRot += 5.0f;
	if(key == GLUT_KEY_LEFT) yRot -= 5.0f;
	if(key == GLUT_KEY_RIGHT) yRot += 5.0f;
	if(key > 356.0f) xRot = 0.0f;
	if(key < -1.0f) xRot = 355.0f;
	if(key > 356.0f) yRot = 0.0f;
	if(key < -1.0f) yRot = 355.0f;

	// Refresh the Window
	glutPostRedisplay();
	*/
}

void ChangeSize(int w, int h)
{
	float nRange = 100.0f;

	// Prevent a divide by zero
	if(h == 0) h = 1;

	// Set Viewport to window dimensions
	glViewport(0, 0, w, h);

	// Reset projection matrix stack
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// Establish clipping volume (left, right, bottom, top, near, far)
	if (w <= h)  glOrtho (-nRange, nRange, -nRange*h/w, nRange*h/w, -nRange, nRange);
	else  glOrtho (-nRange*w/h, nRange*w/h, -nRange, nRange, -nRange, nRange);

	// Reset Model view matrix stack
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

// Main program entry point
int main(int argc, char* argv[])
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
	glutInitWindowSize(DEMO_WIN_SIZE);
	glutCreateWindow(DEMO_TITLE);
	glutReshapeFunc(ChangeSize);
	glutDisplayFunc(render);
	glutSpecialFunc(SpecialKeys);
	glutTimerFunc(1000/DEMO_FPS, TimerFunction, 1);
	glClearColor(DEMO_CLEAR_COLOR);

	init();

	glutMainLoop();

	return 0;
}

#endif

